@@ -57,7 +57,7 @@ public:
     virtual unsigned long length() const;
     CSSRuleImpl *parentRule() const;
     virtual void removeProperty(int propertyID, DOM::DOMString* old = 0);
-    virtual void removePropertiesInSet(const int* set, unsigned length);
+    virtual bool removePropertiesInSet(const int* set, unsigned length);
     virtual bool setProperty ( int propertyId, const DOM::DOMString &value, bool important, int &ec);
     virtual bool setProperty ( int propertyId, const DOM::DOMString &value, bool important = false);
     virtual void setProperty ( int propertyId, int value, bool important = false);
@@ -171,7 +171,7 @@ public:
     virtual ~CSSValueListImpl();
 
     unsigned long length() const { return m_values.count(); }
-    CSSValueImpl *item ( unsigned long index ) { return m_values.at(index); }
+    CSSValueImpl *item ( unsigned long index ) { return  index < length() ? m_values.at(index) : 0; }
 
     virtual bool isValueList() const { return true; }
 
@@ -222,6 +222,16 @@ public:
 
     double computeLengthFloat( khtml::RenderStyle *style, int logicalDpiY);
 
+    /*
+     * rounds a computer value into integer as appropriate. This takes
+     * care of the various 4.9999999999 type cases
+     */
+    static int snapValue( double result ) {
+        // Conversions are imprecise, often resulting in values of, e.g., 44.99998.  We
+        // need to go ahead and round if we're really close to the next integer value.
+        return (int)(result + (result < 0 ? -0.01 : +0.01));
+    }
+
     // Retrieves an explicit resolution from the CSSValue if it contains one.
     // This is specific to the CSS3 Media Queries module's resolution feature.
     int getDPIResolution() const;
@@ -351,7 +361,7 @@ protected:
 class CSSImageValueImpl : public CSSPrimitiveValueImpl, public khtml::CachedObjectClient
 {
 public:
-    CSSImageValueImpl(const DOMString &url, const StyleBaseImpl *style);
+    CSSImageValueImpl(const DOMString &url, StyleBaseImpl *style);
     CSSImageValueImpl();
     virtual ~CSSImageValueImpl();
 
